﻿using System;
using System.Diagnostics;
using System.Linq;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Media;
using Steema.TeeChart.WPF.Functions;
using Steema.TeeChart.WPF.Styles;

namespace TeeChartHistogram
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        FastLine series = new FastLine();

        public MainWindow()
        {
            InitializeComponent();
            series.Visible = false;
            series.Title = Guid.Empty.ToString();
            AddValuesToFastLine(series);
        
            Stopwatch stopWatch = new Stopwatch();
            stopWatch.Start();

            HistogramFunction hist = new HistogramFunction(MyTChart.Chart) { NumBins = 100, IncludeNulls = true, Cumulative = false };

            Points series2 = new Points(MyTChart.Chart);
            series2.Marks.Visible = false;
            series2.DataSource = series;
            //while debugging you can see next line takes more than 2 min time to get execute.
            series2.Function = hist;
            series2.Color = Colors.Red;
            series2.Title = "Test";
            series2.Visible = true;

            ConvertYToPercent(series2);

            FastLine series3 = new FastLine(MyTChart.Chart);
            series3.Marks.Visible = false;
            series3.DataSource =  series2;
            series3.Function = new Smoothing();
            series3.Color = Colors.Blue;
            series3.Title = "Test1";
            series3.Visible = true;

            stopWatch.Stop();

            lblChartGenereted.Content= $"Chart is ready time taken { stopWatch.Elapsed} ";
        }

        /// <summary>
        /// Adds the values to fast line.
        /// </summary>
        /// <param name="plotObject">The plot object.</param>
        /// <param name="series">The series.</param>
        private static void AddValuesToFastLine(FastLine series)
        {

            Task thread1 = Task.Factory.StartNew(() => AddValue(102721, series));
            Task thread2 = Task.Factory.StartNew(() => AddValue(102721, series));
            Task thread3 = Task.Factory.StartNew(() => AddValue(102721, series));

            Task.WaitAll(thread1, thread2, thread3);
            
        }

        private static readonly object balanceLock = new object();
        static void AddValue(int count, FastLine series)
        {
            for (int i = 0; i <= count; i++)
            {
                Random rand = new Random();
                double x1 = NextFloat(42109, 42108);
                double y1 = NextFloat(365, 364);
                lock (balanceLock)
                {
                    series.Add(x1, y1, Colors.Red);
                }
            }
        }

        static double NextFloat(float min, float max)
        {
            System.Random random = new System.Random();
            double val = (random.NextDouble() * (max - min) + min);
            return (float)val;
        }

        private void ConvertYToPercent(Points series2)
        {
            if (series2.YValues != null && series2.YValues.Value != null && series2.YValues.Value.Length > 0)
            {
                double total = series2.YValues.Value.Sum();
                series2.YValues.Value = (from yValve in series2.YValues.Value select (yValve / total) * 100).ToArray();
            }
        }

       

        
    }
}
